import java.net.*;
import java.io.*;

/**
 * A Server-Client Protocol for a ChatRoom.
 */
public class ChatRoomProtocol {
	
	public static final int REQUESTING_CONNECTION = 0;
	public static final int SEND_USER_NAME = 1;
	public static final int USER_NAME = 2;
	public static final int USER_DISCONNECTING = 3;
	public static final int CHAT_ROOM_MESSAGE = 4;
	public static final int WELCOME_MESSAGE = 5;
		
	// single instance of stateless protocol
	private static ChatRoomProtocol instance;
	
	/**
	 * Constructor private.
	 * @pattern singleton
	 */
	private ChatRoomProtocol () {
	}
	
	/**
	 * Used to access the single protocol instance
	 */
	public static ChatRoomProtocol getInstance () {
		if (instance == null) {
			instance = new ChatRoomProtocol();
		}
		return instance;
	}
	
	/**
	 * Process the input string and returns the reply value needed for
	 * the conversation.
	 * @param	input	the input String to process.
	 */
	private int processInput(String input) throws ProtocolException{
		int inputID = CHAT_ROOM_MESSAGE;
		try {
			inputID= Integer.parseInt(input.substring(0,1));
		} catch (NumberFormatException nfe) {
			throw new ProtocolException("Invalid protocol call");
		}
		switch (inputID) {
			case REQUESTING_CONNECTION:
				return SEND_USER_NAME;
			case SEND_USER_NAME:
				return USER_NAME;
			case USER_NAME:
				return WELCOME_MESSAGE;
			case USER_DISCONNECTING:
				return USER_DISCONNECTING;
			default:
				return CHAT_ROOM_MESSAGE;
		}
	}
	
	/**
	 * Creates a connection Message for a chat room client to send to
	 * it's server
	 */
	public String createConnectionRequest() {
		return String.valueOf(REQUESTING_CONNECTION);
	}
	
	/**
	 * Creates a chat client disconnection request message.
	 * @return	A disconnection request string
	 */
	public String createDisconnectionRequest(String userName) {
		return USER_DISCONNECTING + userName;
	}
	
	/**
	 * Creates a chat User-Name response message.
	 * @param	userName	the user name
	 * @return	A user-name response string
	 */
	public String createUserNameResponse(String userName) {
		return USER_NAME + userName;
	}
	
	/**
	 * Creates a regular chat message in the form of: ID<user> message...
	 * The ID is the message type constant.
	 * @param	message	the content of the message
	 * @return	A string of a regular chat message
	 */
	public String createChatMessage(String message) {
		return CHAT_ROOM_MESSAGE + message;
	}
	
	/**
	 * Handles a message received from the client on a given chat room connection.
	 * @param	message	the client's message.
	 * @param	connection	the ChatRoomConnection of that client.
	 */
	public void handleClientMessage (String message,
									 ChatRoomConnection connection) throws IOException {
		switch (processInput(message)) {
			
			case ChatRoomProtocol.SEND_USER_NAME:
				connection.sendMessage(String.valueOf(SEND_USER_NAME));
				break;
			case ChatRoomProtocol.USER_DISCONNECTING:
				connection.close();
				break;
			case ChatRoomProtocol.WELCOME_MESSAGE:
				String userName = message.substring(1);
				connection.setUserName(userName);
				connection.getServer().sendToAllUsers(createChatMessage(userName+ " entered the chat room."));
				break;
			case ChatRoomProtocol.CHAT_ROOM_MESSAGE:
				connection.getServer().sendToAllUsers(createChatMessage(message.substring(1)));
				break;
		}
	}
	
	/**
	 * Handles a message received from the server to a given client.
	 * @param	message	the server's message
	 * @param	client	the ChatRoomClient to communicate with.
	 */
	public void handleServerMessage (String message, ChatRoomClient client)
		throws IOException, ProtocolException {
		int response = processInput(message);
		switch(response) {
			case USER_NAME:
				client.requestUserName();
				break;
			case ChatRoomProtocol.CHAT_ROOM_MESSAGE:
				client.displayMessage(message.substring(1));
		}
	}
}
